-- Auch an Quests, AH, etc. (also Freitext!) erinnern

RefillRemind = LibStub("AceAddon-3.0"):NewAddon("RefillRemind", "AceConsole-3.0", "AceEvent-3.0")
LibStub("AceAddon-3.0"):EmbedLibrary(RefillRemind, "LibFuBarPlugin-3.0", true)

local RefillRemind = RefillRemind
local L = LibStub("AceLocale-3.0"):GetLocale("RefillRemind")

local PT = AceLibrary("LibPeriodicTable-3.1")
local Tourist = Rock("LibTourist-3.0")

-- Got this from FuBar_PTReagents
local additionalSets = {
	WARLOCK = {"Consumable.Warlock.Healthstone", "Consumable.Warlock.Spellstone", "Consumable.Warlock.Soulstone", "Consumable.Warlock.Firestone"},
	ROGUE = {"Consumable.Potion.Other.Energy", "Consumable.Weapon Buff.Poison.Anesthetic", "Consumable.Weapon Buff.Poison.Wound", "Consumable.Weapon Buff.Poison.Mind Numbing", "Consumable.Weapon Buff.Poison.Instant", "Consumable.Weapon Buff.Poison.Crippling", "Consumable.Weapon Buff.Poison.Deadly"},
	MAGE = {"Consumable.Mage.Mana Stone"},
	HUNTER = {"Misc.Reagent.Ammo.Bullet", "Misc.Reagent.Ammo.Arrow"},
	WARRIOR = {"Misc.Reagent.Ammo.Bullet", "Misc.Reagent.Ammo.Arrow"},
}

-- Options
local options = {
    name = "RefillRemind",
    handler = RefillRemind,
    type = "group",
    args = {
		watchedItems = {
			type = "group",
			name = L["Watched items"],
			desc = L["Watched items"],
			args = {
			}
		},
		fubar = {
			type = "group",
			name = L["FuBar options"],
			desc = L["FuBar options"],
			disabled = function() return not RefillRemind.IsFuBarMinimapAttached end,
			hidden = function() return not RefillRemind.IsFuBarMinimapAttached end,
			order = 100,
			args = {
				attachMinimap = {
					type = "toggle",
					name = L["Attach to minimap"],
					desc = L["Attach to minimap"],
					get = function(info)
						return RefillRemind:IsFuBarMinimapAttached()
					end,
					set = function(info, v)
						RefillRemind:ToggleFuBarMinimapAttached()
						RefillRemind.db.char.FuBar_AttachMinimap = RefillRemind:IsFuBarMinimapAttached()
					end
				},
				hideIcon = {
					type = "toggle",
					name = L["Hide minimap/FuBar icon"],
					desc = L["Hide minimap/FuBar icon"],
					get = function(info) return RefillRemind.db.char.FuBar_HideMinimapButton end,
					set = function(info, v)
						RefillRemind.db.char.FuBar_HideMinimapButton = v
						if v then
							RefillRemind:Hide()
						else
							RefillRemind:Show()
						end
					end
				},
				showIcon = {
					type = "toggle",
					name = L["Show icon"],
					desc = L["Show icon"],
					get = function(info) return RefillRemind:IsFuBarIconShown() end,
					set = function(info, v) RefillRemind:ToggleFuBarIconShown() end
				},
				showText = {
					type = "toggle",
					name = L["Show text"],
					desc = L["Show text"],
					get = function(info) return RefillRemind:IsFuBarTextShown() end,
					set = function(info, v) RefillRemind:ToggleFuBarTextShown() end
				},
				position = {
					type = "select",
					name = L["Position"],
					desc = L["Position"],
					values = {LEFT = L["Left"], CENTER = L["Center"], RIGHT = L["Right"]},
					get = function() return RefillRemind:GetPanel() and RefillRemind:GetPanel():GetPluginSide(RefillRemind) end,
					set = function(info, val)
						if RefillRemind:GetPanel() then
							RefillRemind:GetPanel():SetPluginSide(RefillRemind, val)
						end
					end
				}
			}
		},
    },
}

-- Default options
local defaults = {
    char = {
        importedDefaults = false,
		watchedItems = {}
    },
}

local optionsFrame

function RefillRemind:OnInitialize()
	-- Called when the addon is loaded
	RefillRemind:SetFuBarOption("iconPath",[[Interface\Icons\INV_Misc_Book_09]])
	RefillRemind:SetFuBarOption("defaultPosition","LEFT")
	RefillRemind:SetFuBarOption("hideWithoutStandby",true)
	
	self.db = LibStub("AceDB-3.0"):New("RefillRemindDB", defaults, "char")
	if not self.db.importedDefaults then
		self:ImportDefaults()
	end
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("RefillRemind", options)
    self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("RefillRemind", "RefillRemind")
    self:RegisterChatCommand("rr", "ChatCommand")
    self:RegisterChatCommand("refillremind", "ChatCommand")
end

function RefillRemind:ImportDefaults()
	-- Import the default items for the player class
	local _, playerClass = UnitClass("player")
	local class = playerClass:lower():gsub("^%l", string.upper)
	
	local ptSet = PT:GetSetTable("Misc.Reagent.Class."..class)
	if type(ptSet) == "table" then
		for k, v in pairs(ptSet) do
			if k ~= "set" then
				self:AddItem(k, "", "default")
			end
		end
	end

	local itemSets = {}
	if type(additionalSets[playerClass]) == "table" then
		for i, v in ipairs(additionalSets[playerClass]) do
			for k, v in pairs(PT:GetSetTable(v)) do
				self:AddItem(k, "", "default")
			end
		end
	end
end

function RefillRemind:AddItem(itemId, displayName, entryType)
	-- Adds an item to the watched items
	if displayName == "" then
		displayName = GetItemInfo(itemId)
		if not displayName then
			displayName = L["Unknown"].." "..itemId
		end
	end
	
	local newItem = {
		displayName = displayName,
		entryType = entryType
	}
	
	RefillRemind.db.char.watchedItems[itemId] = newItem
	RefillRemind.db.char.show = true
end

function RefillRemind:ChatCommand(input)
	RefillRemind:RebuildConfig()
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToFrame(self.optionsFrame)
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(RefillRemind, "refillremind", "RefillRemind", input)
    end
end

function RefillRemind:OnEnable()
    self:RegisterEvent("PLAYER_UPDATE_RESTING")
    --self:RegisterEvent("MERCHANT_SHOW")
    --self:RegisterEvent("MERCHANT_CLOSED")
    --self:RegisterEvent("BAG_UPDATE")
    --self:RegisterEvent("BANKFRAME_OPENED")
    --self:RegisterEvent("BANKFRAME_CLOSED")
end

function RefillRemind:PLAYER_UPDATE_RESTING()
	-- Only start of resting is interesting for us
	if not IsResting() then return end
	
	-- Check whether we are in a major city or not
	--if Tourist:IsCity(GetZoneText()) then
	
	--end
end

function RefillRemind:OnDisable()
    -- Events are de-registered by Ace3
end

StaticPopupDialogs["RefillRemind_BadItem"] = {
	text = L["Unknown item."],
	button1 = OKAY,
	showAlert = 1,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
}

StaticPopupDialogs["RefillRemind_AskItemName"] = {
	text = L["Please input the name or the ItemID of the item you want to add (using ItemID could cause connection aborts)"],
	button1 = OKAY,
	button2 = CANCEL,
	OnShow = function()
		getglobal(this:GetName().."EditBox"):SetText("")
	end,
	OnAccept = function()
		local input = getglobal(this:GetParent():GetName().."EditBox"):GetText()
		local itemId = tonumber(input)
		if itemId then
			-- Check ItemID
			local displayName = GetItemInfo(input)
			if displayName then
				RefillRemind:AddItem(itemId, displayName, "custom")
				RefillRemind:RebuildConfig()
				return false
			else
				StaticPopup_Show("RefillRemind_BadItem")
				return true
			end
		else
			-- Check ItemName
			local itemNameOkay = false
			local itemLink = select(2, GetItemInfo(input))
			if itemLink then
				local found, _, itemString = string.find(itemLink, "^|c%x+|H(.+)|h%[.+%]")
				if itemString then
					local foundItemID = select(2, strsplit(":", itemString))
					if foundItemID then
						itemId = foundItemID
						itemNameOkay = true
					end
				end
			end
			-- And add if okay
			if itemNameOkay then
				RefillRemind:AddItem(itemId, input, "custom")
				RefillRemind:RebuildConfig()
				return false
			else
				StaticPopup_Show("RefillRemind_BadItem")
				return true
			end
		end
	end,
	hasEditBox = 1,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};

function RefillRemind:RebuildConfig()
	-- Rebuilds the config submenus
	options.args.watchedItems.args = {
		removeAll = {
			type = "execute",
			name = L["Remove all"],
			desc = L["Remove all"],
			confirm = true,
			func = function()
				RefillRemind.db.char.watchedItems = {}
				RefillRemind:RebuildConfig()
			end
		},
		addNew = {
			type = "execute",
			name = L["Add new item"],
			desc = L["Add new item"],
			--input = true, -- allowed (according docu) but throws error!?
			func = function()
				StaticPopup_Show ("RefillRemind_AskItemName");
			end
		}
	}
	
	-- Add submenu for every watched item
	for itemId, v in pairs(RefillRemind.db.char.watchedItems) do
		options.args.watchedItems.args["item"..itemId] = {
			type = "group",
			name = v.displayName,
			desc = v.displayName,
			args = {
				show = {
					type = "toggle",
					name = L["Show in FuBar tooltip"],
					desc = L["Show in FuBar tooltip"],
					get = function(info) return RefillRemind.db.char.watchedItems[itemId].show end,
					set = function(info, newVal) RefillRemind.db.char.watchedItems[itemId].show = newVal end
				},
				delete = {
					type = "execute",
					name = L["Remove"],
					desc = L["Remove"],
					func = function()
						options.args.watchedItems.args["item"..itemId] = nil
						RefillRemind.db.char.watchedItems[itemId] = nil
					end
				}
			}
		}
	end
end

--[[===============================================
=================== FuBar Stuff ===================
=================================================--]]

function RefillRemind:OpenMenu()
	self:ChatCommand("")
end

function RefillRemind:OnUpdateFuBarText()
	RefillRemind:SetFuBarText("RR")
end

function RefillRemind:OnUpdateFuBarTooltip()
	GameTooltip:ClearLines()
	GameTooltip:AddLine("|cffffff00"..L["RefillRemind"].."|r")
	GameTooltip:AddLine(L["|cffffff00Right-click|r to open the Configuration GUI"])
	
	for itemId, v in pairs(RefillRemind.db.char.watchedItems) do
		local itemCount = GetItemCount(itemId)
		GameTooltip:AddLine(itemCount.." x "..v.displayName)
	end
end
